package ru.dbotthepony.mc.otm.android.feature

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import com.mojang.math.Quaternion
import com.mojang.math.Vector3f
import net.minecraft.ChatFormatting
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.ClipContext
import net.minecraft.world.level.block.Block
import net.minecraft.world.phys.BlockHitResult
import net.minecraft.world.phys.HitResult
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraftforge.client.event.RenderLevelStageEvent
import org.lwjgl.opengl.GL11.GL_LESS
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.ServerConfig
import ru.dbotthepony.mc.otm.android.AndroidActiveFeature
import ru.dbotthepony.mc.otm.android.AndroidResearchManager
import ru.dbotthepony.mc.otm.capability.MatteryPlayerCapability
import ru.dbotthepony.mc.otm.capability.extractEnergyInnerExact
import ru.dbotthepony.mc.otm.client.render.ResearchIcons
import ru.dbotthepony.mc.otm.client.render.element
import ru.dbotthepony.mc.otm.core.Vector
import ru.dbotthepony.mc.otm.core.asVector
import ru.dbotthepony.mc.otm.core.component1
import ru.dbotthepony.mc.otm.core.component2
import ru.dbotthepony.mc.otm.core.component3
import ru.dbotthepony.mc.otm.core.formatPower
import ru.dbotthepony.mc.otm.core.genericPositions
import ru.dbotthepony.mc.otm.core.plus
import ru.dbotthepony.mc.otm.core.shortestDistanceBetween
import ru.dbotthepony.mc.otm.core.times
import ru.dbotthepony.mc.otm.registry.AndroidFeatures
import ru.dbotthepony.mc.otm.registry.MNames
import kotlin.math.sin

class EnderTeleporterFeature(capability: MatteryPlayerCapability) : AndroidActiveFeature(AndroidFeatures.ENDER_TELEPORTER, capability) {
	var cooldown by synchronizer.int()

	private fun canUse(): Boolean {
		return cooldown <= 0 && android.androidEnergy.extractEnergyInnerExact(ServerConfig.EnderTeleporter.ENERGY_COST, true).isPositive
	}

	private fun isValidPosition(blockPos: BlockPos): Boolean {
		if (!Block.canSupportCenter(ply.level, blockPos.below(), Direction.UP) && !Block.canSupportRigidBlock(ply.level, blockPos.below())) {
			return false
		}

		val a = ply.level.getBlockState(blockPos)
		val b = ply.level.getBlockState(blockPos.above())

		if (a.isAir && b.isAir) {
			return true
		}

		if (a.getCollisionShape(ply.level, blockPos).isEmpty && b.getCollisionShape(ply.level, blockPos, CollisionContext.of(ply)).isEmpty) {
			return true
		}

		return false
	}

	private fun trace(): BlockPos? {
		val headPosition = ply.eyePosition
		val aimVector = ply.getViewVector(1f)

		val result = ply.level.clip(ClipContext(
			headPosition,
			headPosition + aimVector * (ServerConfig.EnderTeleporter.MAX_DISTANCE * 2.0),
			ClipContext.Block.COLLIDER,
			ClipContext.Fluid.NONE,
			ply
		))

		if (result.isInside || result.type == HitResult.Type.MISS) {
			return null
		}

		if (
			result.direction == Direction.UP &&
			isValidPosition(result.blockPos.above()) &&
			shortestDistanceBetween(ply.genericPositions(), result.blockPos.above().asVector()) <= ServerConfig.EnderTeleporter.MAX_DISTANCE
		) {
			return result.blockPos.above()
		}

		for (y in 0 .. ServerConfig.EnderTeleporter.MAX_PHASE_DISTANCE) {
			val pos = result.blockPos + BlockPos(0, y + 1, 0)

			if (isValidPosition(pos) && shortestDistanceBetween(ply.genericPositions(), pos.asVector()) <= ServerConfig.EnderTeleporter.MAX_DISTANCE) {
				return pos
			}
		}

		return null
	}

	override fun activate(isClient: Boolean): Boolean {
		if (!canUse()) {
			return false
		}

		val blockPos = trace() ?: return false

		cooldown = ServerConfig.EnderTeleporter.COOLDOWN

		if (!isClient) android.androidEnergy.extractEnergyInner(ServerConfig.EnderTeleporter.ENERGY_COST, false)
		ply.setPos(Vector(blockPos.x + 0.5, blockPos.y.toDouble(), blockPos.z + 0.5))
		ply.deltaMovement = Vector(0.0, 0.0, 0.0)
		ply.resetFallDistance()
		return true
	}

	override fun tickServer() {
		if (cooldown > 0) {
			cooldown--
		}
	}

	override fun renderPreview(event: RenderLevelStageEvent) {
		if (!canUse()) return
		val pos = trace()

		if (pos != null) {
			val (x, y, z) = pos.asVector()
			val (vx, vy, vz) = event.camera.position
			//event.poseStack.translate(x, y, z)

			event.poseStack.translate(x - vx, y - vy, z - vz)
			event.poseStack.mulPose(Vector3f.YP.rotationDegrees(-event.camera.yRot))
			event.poseStack.mulPose(Vector3f.XP.rotationDegrees(event.camera.xRot))

			val size = 1f + sin((System.nanoTime() / 1_000_000) / 250.0).toFloat() * 0.2f
			val half = size / -2f

			RenderSystem.enableDepthTest()
			RenderSystem.depthFunc(GL_LESS)

			RenderSystem.setShaderColor(1f, 1f, 1f, 0.5f)
			SPRITE.render(event.poseStack, x = half, y = half, width = size, height = size)
			RenderSystem.setShaderColor(1f, 1f, 1f, 1f)
		}
	}

	override fun renderIcon(stack: PoseStack, x: Float, y: Float, width: Float, height: Float) {
		if (cooldown > 0) {
			RenderSystem.setShaderColor(1f, 0.4f, 0.4f, 1f)
		}

		ResearchIcons.ICON_ENDER_TELEPORT.render(stack, x, y, width, height)

		if (cooldown > 0) {
			RenderSystem.setShaderColor(1f, 1f, 1f, 1f)
		}
	}

	companion object {
		val SPRITE = ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/item/black_hole.png").element(0f, 0f, 16f, 16f, 16f, 16f)

		val POWER_COST_DESCRIPTION =
			AndroidResearchManager.descriptionFunc(
				ResourceLocation(OverdriveThatMatters.MOD_ID, MNames.ENDER_TELEPORTER),
				"otm.gui.power_cost_per_use",
				{ ServerConfig.EnderTeleporter.ENERGY_COST.formatPower().copy().withStyle(ChatFormatting.YELLOW) })
	}
}
