package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.material.Material
import net.minecraft.world.level.material.MaterialColor
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.block.entity.EnergyServoBlockEntity
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.shapes.BlockShapes

class EnergyServoBlock : RotatableMatteryBlock(Properties.of(Material.METAL, MaterialColor.COLOR_BLUE).explosionResistance(12f).destroyTime(2f)), EntityBlock {
	override fun newBlockEntity(p_153215_: BlockPos, p_153216_: BlockState): BlockEntity {
		return EnergyServoBlockEntity(p_153215_, p_153216_)
	}

	override fun <T : BlockEntity?> getTicker(
		p_153212_: Level,
		p_153213_: BlockState,
		p_153214_: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (p_153214_ == MBlockEntities.ENERGY_SERVO && !p_153212_.isClientSide) {
			return BlockEntityTicker { _, _, _, it -> if (it is EnergyServoBlockEntity) it.tick() }
		}

		return null
	}

	override fun getShape(
			p_60555_: BlockState,
			p_60556_: BlockGetter,
			p_60557_: BlockPos,
			p_60558_: CollisionContext
	): VoxelShape {
		return SHAPES[p_60555_.getValue(FACING).ordinal]
	}

	companion object {
		private val SHAPES: Array<VoxelShape>

		init {
			val def = BlockShapes.ENERGY_SERVO.computeShape()

			SHAPES = arrayOf(
					def,
					def,
					def,
					BlockShapes.ENERGY_SERVO.rotate(Direction.NORTH).computeShape(),
					BlockShapes.ENERGY_SERVO.rotate(Direction.WEST).computeShape(),
					BlockShapes.ENERGY_SERVO.rotate(Direction.EAST).computeShape()
			)
		}
	}
}
