package ru.dbotthepony.mc.otm.core

class TimerQueue {
	private var ticks = 0
	private var head: Timer? = null

	inner class Timer(val timerTicks: Int, val runnable: Runnable) {
		val ringAt = ticks + timerTicks
		var finished = false
			private set

		var next: Timer? = null
		var prev: Timer? = null

		init {
			if (head == null) {
				head = this
			} else {
				var next = head

				while (next != null) {
					if (next.ringAt >= this.ringAt) {
						next.prev?.next = this
						this.prev = next.prev
						next.prev = this
						this.next = next
						break
					} else if (next.next == null) {
						next.next = this
						this.prev = next
						break
					} else {
						next = next.next
					}
				}
			}
		}

		fun execute() {
			check(!finished) { "Already finished" }
			runnable.run()
			finished = true
		}
	}

	fun tick() {
		ticks++

		var head = head

		while (head != null) {
			if (head.ringAt <= ticks) {
				head.execute()
				head = head.next
				head?.prev = null
				this.head = head
			} else {
				break
			}
		}
	}

	fun clear() {
		ticks = 0
		head = null
	}
}