package ru.dbotthepony.mc.otm.menu

import com.google.common.collect.ImmutableList
import kotlin.jvm.JvmOverloads
import net.minecraft.world.entity.player.Inventory
import ru.dbotthepony.mc.otm.block.entity.matter.MatterDecomposerBlockEntity
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import net.minecraft.world.SimpleContainer
import net.minecraft.world.inventory.Slot
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.matter.canDecompose
import ru.dbotthepony.mc.otm.core.orNull
import ru.dbotthepony.mc.otm.registry.MMenus

class MatterDecomposerMenu @JvmOverloads constructor(
	containerID: Int,
	inventory: Inventory,
	tile: MatterDecomposerBlockEntity? = null
) : MatteryPoweredMenu(MMenus.MATTER_DECOMPOSER, containerID, inventory, tile) {
	val input: MatterySlot
	val outputMain: MachineOutputSlot
	val outputStacking: MachineOutputSlot
	val progressWidget: ProgressGaugeWidget
	val matterWidget: LevelGaugeWidget

	init {
		val container = tile?.container ?: SimpleContainer(3)

		// Вход
		input = object : MatterySlot(container, 0) {
			override fun mayPlace(p_40231_: ItemStack) = canDecompose(p_40231_)
		}

		// Выход
		outputMain = MachineOutputSlot(container, 1)
		outputStacking = MachineOutputSlot(container, 2)

		matterWidget = LevelGaugeWidget(this, tile?.getCapability(MatteryCapability.MATTER)?.orNull())

		if (tile == null) {
			progressWidget = ProgressGaugeWidget(this)
		} else {
			progressWidget = ProgressGaugeWidget(this, tile::workProgress, tile::isUnableToProcess)
		}

		addInventorySlots()
	}

	override val storageSlots: List<Slot> = ImmutableList.of(
		addSlot(outputMain),
		addSlot(outputStacking),
		addSlot(input),
	)
}
